@@ -1,9 +1,9 @@
-from __future__ import annotations
-
+import dataclasses
 import datetime
 import json
 import logging
 import re
+import types
 from dataclasses import asdict, dataclass, field
 from datetime import timezone
 from enum import Enum
@@ -11,6 +11,7 @@ from functools import cached_property
 from typing import Any, NamedTuple, get_args, get_origin
 
 from .code_mappings import (
+    SHORT_MODEL_TO_ENUM,
     RoborockCategory,
     RoborockCleanType,
     RoborockDockDustCollectionModeCode,
@@ -28,8 +29,11 @@ from .code_mappings import (
     RoborockFanSpeedS7,
     RoborockFanSpeedS7MaxV,
     RoborockFanSpeedS8MaxVUltra,
+    RoborockFanSpeedSaros10,
+    RoborockFanSpeedSaros10R,
     RoborockFinishReason,
     RoborockInCleaning,
+    RoborockModeEnum,
     RoborockMopIntensityCode,
     RoborockMopIntensityP10,
     RoborockMopIntensityQ7Max,
@@ -40,6 +44,8 @@ from .code_mappings import (
     RoborockMopIntensityS6MaxV,
     RoborockMopIntensityS7,
     RoborockMopIntensityS8MaxVUltra,
+    RoborockMopIntensitySaros10,
+    RoborockMopIntensitySaros10R,
     RoborockMopModeCode,
     RoborockMopModeQRevoCurv,
     RoborockMopModeQRevoMaster,
@@ -47,6 +53,9 @@ from .code_mappings import (
     RoborockMopModeS7,
     RoborockMopModeS8MaxVUltra,
     RoborockMopModeS8ProUltra,
+    RoborockMopModeSaros10,
+    RoborockMopModeSaros10R,
+    RoborockProductNickname,
     RoborockStartType,
     RoborockStateCode,
 )
@@ -56,6 +65,7 @@ from .const import (
     FILTER_REPLACE_TIME,
     MAIN_BRUSH_REPLACE_TIME,
     MOP_ROLLER_REPLACE_TIME,
+    NO_MAP,
     ROBOROCK_G10S_PRO,
     ROBOROCK_P10,
     ROBOROCK_Q7_MAX,
@@ -74,115 +84,110 @@ from .const import (
     ROBOROCK_S8,
     ROBOROCK_S8_MAXV_ULTRA,
     ROBOROCK_S8_PRO_ULTRA,
+    ROBOROCK_SAROS_10,
+    ROBOROCK_SAROS_10R,
     SENSOR_DIRTY_REPLACE_TIME,
     SIDE_BRUSH_REPLACE_TIME,
     STRAINER_REPLACE_TIME,
     ROBOROCK_G20S_Ultra,
 )
+from .device_features import DeviceFeatures
 from .exceptions import RoborockException
 
 _LOGGER = logging.getLogger(__name__)
 
 
-def camelize(s: str):
+def _camelize(s: str):
     first, *others = s.split("_")
     if len(others) == 0:
         return s
     return "".join([first.lower(), *map(str.title, others)])
 
 
-def decamelize(s: str):
+def _decamelize(s: str):
     return re.sub("([A-Z]+)", "_\\1", s).lower()
 
 
-def decamelize_obj(d: dict | list, ignore_keys: list[str]):
-    if isinstance(d, RoborockBase):
-        d = d.as_dict()
-    if isinstance(d, list):
-        return [decamelize_obj(i, ignore_keys) if isinstance(i, dict | list) else i for i in d]
-    return {
-        (decamelize(a) if a not in ignore_keys else a): decamelize_obj(b, ignore_keys)
-        if isinstance(b, dict | list)
-        else b
-        for a, b in d.items()
-    }
+def _attr_repr(obj: Any) -> str:
+    """Return a string representation of the object including specified attributes.
+
+    This reproduces the default repr behavior of dataclasses, but also includes
+    properties. This must be called by the child class's __repr__ method since
+    the parent RoborockBase class does not know about the child class's attributes.
+    """
+    # Reproduce default repr behavior
+    parts = []
+    for k in dir(obj):
+        if k.startswith("_"):
+            continue
+        try:
+            v = getattr(obj, k)
+        except (RuntimeError, Exception):
+            continue
+        if callable(v):
+            continue
+        parts.append(f"{k}={v!r}")
+    return f"{type(obj).__name__}({', '.join(parts)})"
 
 
-@dataclass
+@dataclass(repr=False)
 class RoborockBase:
-    _ignore_keys = []  # type: ignore
-    is_cached = False
+    """Base class for all Roborock data classes."""
 
     @staticmethod
-    def convert_to_class_obj(type, value):
-        try:
-            class_type = eval(type)
-            if get_origin(class_type) is list:
-                return_list = []
-                cls_type = get_args(class_type)[0]
-                for obj in value:
-                    if issubclass(cls_type, RoborockBase):
-                        return_list.append(cls_type.from_dict(obj))
-                    elif cls_type in {str, int, float}:
-                        return_list.append(cls_type(obj))
-                    else:
-                        return_list.append(cls_type(**obj))
-                return return_list
-            if issubclass(class_type, RoborockBase):
-                converted_value = class_type.from_dict(value)
-            else:
-                converted_value = class_type(value)
-            return converted_value
-        except NameError as err:
-            _LOGGER.exception(err)
-        except ValueError as err:
-            _LOGGER.exception(err)
-        except Exception as err:
-            _LOGGER.exception(err)
-        raise Exception("Fail")
+    def _convert_to_class_obj(class_type: type, value):
+        if get_origin(class_type) is list:
+            sub_type = get_args(class_type)[0]
+            return [RoborockBase._convert_to_class_obj(sub_type, obj) for obj in value]
+        if get_origin(class_type) is dict:
+            _, value_type = get_args(class_type)  # assume keys are only basic types
+            return {k: RoborockBase._convert_to_class_obj(value_type, v) for k, v in value.items()}
+        if issubclass(class_type, RoborockBase):
+            return class_type.from_dict(value)
+        if issubclass(class_type, RoborockModeEnum):
+            return class_type.from_code(value)
+        if class_type is Any:
+            return value
+        return class_type(value)  # type: ignore[call-arg]
 
     @classmethod
     def from_dict(cls, data: dict[str, Any]):
-        if isinstance(data, dict):
-            ignore_keys = cls._ignore_keys
-            data = decamelize_obj(data, ignore_keys)
-            cls_annotations: dict[str, str] = {}
-            for base in reversed(cls.__mro__):
-                cls_annotations.update(getattr(base, "__annotations__", {}))
-            remove_keys = []
-            for key, value in data.items():
-                if key not in cls_annotations:
-                    remove_keys.append(key)
-                    continue
-                if value == "None" or value is None:
-                    data[key] = None
-                    continue
-                field_type: str = cls_annotations[key]
-                if "|" in field_type:
-                    # It's a union
-                    types = field_type.split("|")
-                    for type in types:
-                        if "None" in type or "Any" in type:
-                            continue
-                        try:
-                            data[key] = RoborockBase.convert_to_class_obj(type, value)
-                            break
-                        except Exception:
-                            ...
-                else:
+        """Create an instance of the class from a dictionary."""
+        if not isinstance(data, dict):
+            return None
+        field_types = {field.name: field.type for field in dataclasses.fields(cls)}
+        result: dict[str, Any] = {}
+        for orig_key, value in data.items():
+            key = _decamelize(orig_key)
+            if (field_type := field_types.get(key)) is None:
+                continue
+            if value == "None" or value is None:
+                result[key] = None
+                continue
+            if isinstance(field_type, types.UnionType):
+                for subtype in get_args(field_type):
+                    if subtype is types.NoneType:
+                        continue
                     try:
-                        data[key] = RoborockBase.convert_to_class_obj(field_type, value)
+                        result[key] = RoborockBase._convert_to_class_obj(subtype, value)
+                        break
                     except Exception:
-                        ...
-            for key in remove_keys:
-                del data[key]
-            return cls(**data)
+                        _LOGGER.exception(f"Failed to convert {key} with value {value} to type {subtype}")
+                        continue
+            else:
+                try:
+                    result[key] = RoborockBase._convert_to_class_obj(field_type, value)
+                except Exception:
+                    _LOGGER.exception(f"Failed to convert {key} with value {value} to type {field_type}")
+                    continue
+
+        return cls(**result)
 
     def as_dict(self) -> dict:
         return asdict(
             self,
             dict_factory=lambda _fields: {
-                camelize(key): value.value if isinstance(value, Enum) else value
+                _camelize(key): value.value if isinstance(value, Enum) else value
                 for (key, value) in _fields
                 if value is not None
             },
@@ -196,21 +201,26 @@ class RoborockBaseTimer(RoborockBase):
     end_hour: int | None = None
     end_minute: int | None = None
     enabled: int | None = None
-    start_time: datetime.time | None = None
-    end_time: datetime.time | None = None
 
-    def __post_init__(self) -> None:
-        self.start_time = (
+    @property
+    def start_time(self) -> datetime.time | None:
+        return (
             datetime.time(hour=self.start_hour, minute=self.start_minute)
             if self.start_hour is not None and self.start_minute is not None
             else None
         )
-        self.end_time = (
+
+    @property
+    def end_time(self) -> datetime.time | None:
+        return (
             datetime.time(hour=self.end_hour, minute=self.end_minute)
             if self.end_hour is not None and self.end_minute is not None
             else None
         )
 
+    def __repr__(self) -> str:
+        return _attr_repr(self)
+
 
 @dataclass
 class Reference(RoborockBase):
@@ -298,144 +308,6 @@ class HomeDataDevice(RoborockBase):
     silent_ota_switch: bool | None = None
     setting: Any | None = None
     f: bool | None = None
-    device_features: DeviceFeatures | None = None
-
-    # seemingly not just str like I thought - example: '0000000000002000' and '0000000000002F63'
-
-    # def __post_init__(self):
-    #     if self.feature_set is not None and self.new_feature_set is not None and self.new_feature_set != "":
-    #         self.device_features = build_device_features(self.feature_set, self.new_feature_set)
-
-
-@dataclass
-class DeviceFeatures(RoborockBase):
-    map_carpet_add_supported: bool
-    show_clean_finish_reason_supported: bool
-    resegment_supported: bool
-    video_monitor_supported: bool
-    any_state_transit_goto_supported: bool
-    fw_filter_obstacle_supported: bool
-    video_setting_supported: bool
-    ignore_unknown_map_object_supported: bool
-    set_child_supported: bool
-    carpet_supported: bool
-    mop_path_supported: bool
-    multi_map_segment_timer_supported: bool
-    custom_water_box_distance_supported: bool
-    wash_then_charge_cmd_supported: bool
-    room_name_supported: bool
-    current_map_restore_enabled: bool
-    photo_upload_supported: bool
-    shake_mop_set_supported: bool
-    map_beautify_internal_debug_supported: bool
-    new_data_for_clean_history: bool
-    new_data_for_clean_history_detail: bool
-    flow_led_setting_supported: bool
-    dust_collection_setting_supported: bool
-    rpc_retry_supported: bool
-    avoid_collision_supported: bool
-    support_set_switch_map_mode: bool
-    support_smart_scene: bool
-    support_floor_edit: bool
-    support_furniture: bool
-    support_room_tag: bool
-    support_quick_map_builder: bool
-    support_smart_global_clean_with_custom_mode: bool
-    record_allowed: bool
-    careful_slow_map_supported: bool
-    egg_mode_supported: bool
-    unsave_map_reason_supported: bool
-    carpet_show_on_map: bool
-    supported_valley_electricity: bool
-    drying_supported: bool
-    download_test_voice_supported: bool
-    support_backup_map: bool
-    support_custom_mode_in_cleaning: bool
-    support_remote_control_in_call: bool
-    support_set_volume_in_call: bool
-    support_clean_estimate: bool
-    support_custom_dnd: bool
-    carpet_deep_clean_supported: bool
-    stuck_zone_supported: bool
-    custom_door_sill_supported: bool
-    clean_route_fast_mode_supported: bool
-    cliff_zone_supported: bool
-    smart_door_sill_supported: bool
-    support_floor_direction: bool
-    wifi_manage_supported: bool
-    back_charge_auto_wash_supported: bool
-    support_incremental_map: bool
-    offline_map_supported: bool
-
-
-def build_device_features(feature_set: str, new_feature_set: str) -> DeviceFeatures:
-    new_feature_set_int = int(new_feature_set)
-    feature_set_int = int(feature_set)
-    new_feature_set_divided = int(new_feature_set_int / (2**32))
-    # Convert last 8 digits of new feature set into hexadecimal number
-    converted_new_feature_set = int("0x" + new_feature_set[-8:], 16)
-    new_feature_set_mod_8: bool = len(new_feature_set) % 8 == 0
-    return DeviceFeatures(
-        map_carpet_add_supported=bool(1073741824 & new_feature_set_int),
-        show_clean_finish_reason_supported=bool(1 & new_feature_set_int),
-        resegment_supported=bool(4 & new_feature_set_int),
-        video_monitor_supported=bool(8 & new_feature_set_int),
-        any_state_transit_goto_supported=bool(16 & new_feature_set_int),
-        fw_filter_obstacle_supported=bool(32 & new_feature_set_int),
-        video_setting_supported=bool(64 & new_feature_set_int),
-        ignore_unknown_map_object_supported=bool(128 & new_feature_set_int),
-        set_child_supported=bool(256 & new_feature_set_int),
-        carpet_supported=bool(512 & new_feature_set_int),
-        mop_path_supported=bool(2048 & new_feature_set_int),
-        multi_map_segment_timer_supported=bool(feature_set_int and 4096 & new_feature_set_int),
-        custom_water_box_distance_supported=bool(new_feature_set_int and 2147483648 & new_feature_set_int),
-        wash_then_charge_cmd_supported=bool((new_feature_set_divided >> 5) & 1),
-        room_name_supported=bool(16384 & new_feature_set_int),
-        current_map_restore_enabled=bool(8192 & new_feature_set_int),
-        photo_upload_supported=bool(65536 & new_feature_set_int),
-        shake_mop_set_supported=bool(262144 & new_feature_set_int),
-        map_beautify_internal_debug_supported=bool(2097152 & new_feature_set_int),
-        new_data_for_clean_history=bool(4194304 & new_feature_set_int),
-        new_data_for_clean_history_detail=bool(8388608 & new_feature_set_int),
-        flow_led_setting_supported=bool(16777216 & new_feature_set_int),
-        dust_collection_setting_supported=bool(33554432 & new_feature_set_int),
-        rpc_retry_supported=bool(67108864 & new_feature_set_int),
-        avoid_collision_supported=bool(134217728 & new_feature_set_int),
-        support_set_switch_map_mode=bool(268435456 & new_feature_set_int),
-        support_smart_scene=bool(new_feature_set_divided & 2),
-        support_floor_edit=bool(new_feature_set_divided & 8),
-        support_furniture=bool((new_feature_set_divided >> 4) & 1),
-        support_room_tag=bool((new_feature_set_divided >> 6) & 1),
-        support_quick_map_builder=bool((new_feature_set_divided >> 7) & 1),
-        support_smart_global_clean_with_custom_mode=bool((new_feature_set_divided >> 8) & 1),
-        record_allowed=bool(1024 & new_feature_set_int),
-        careful_slow_map_supported=bool((new_feature_set_divided >> 9) & 1),
-        egg_mode_supported=bool((new_feature_set_divided >> 10) & 1),
-        unsave_map_reason_supported=bool((new_feature_set_divided >> 14) & 1),
-        carpet_show_on_map=bool((new_feature_set_divided >> 12) & 1),
-        supported_valley_electricity=bool((new_feature_set_divided >> 13) & 1),
-        # This one could actually be incorrect
-        # ((t.robotNewFeatures / 2 ** 32) >> 15) & 1 && (module422.DMM.isTopazSV_CE || 'cn' == t.deviceLocation));
-        drying_supported=bool((new_feature_set_divided >> 15) & 1),
-        download_test_voice_supported=bool((new_feature_set_divided >> 16) & 1),
-        support_backup_map=bool((new_feature_set_divided >> 17) & 1),
-        support_custom_mode_in_cleaning=bool((new_feature_set_divided >> 18) & 1),
-        support_remote_control_in_call=bool((new_feature_set_divided >> 19) & 1),
-        support_set_volume_in_call=new_feature_set_mod_8 and bool(1 & converted_new_feature_set),
-        support_clean_estimate=new_feature_set_mod_8 and bool(2 & converted_new_feature_set),
-        support_custom_dnd=new_feature_set_mod_8 and bool(4 & converted_new_feature_set),
-        carpet_deep_clean_supported=bool(8 & converted_new_feature_set),
-        stuck_zone_supported=new_feature_set_mod_8 and bool(16 & converted_new_feature_set),
-        custom_door_sill_supported=new_feature_set_mod_8 and bool(32 & converted_new_feature_set),
-        clean_route_fast_mode_supported=bool(256 & converted_new_feature_set),
-        cliff_zone_supported=new_feature_set_mod_8 and bool(512 & converted_new_feature_set),
-        smart_door_sill_supported=new_feature_set_mod_8 and bool(1024 & converted_new_feature_set),
-        support_floor_direction=new_feature_set_mod_8 and bool(2048 & converted_new_feature_set),
-        wifi_manage_supported=bool(128 & converted_new_feature_set),
-        back_charge_auto_wash_supported=bool(4096 & converted_new_feature_set),
-        support_incremental_map=bool(8192 & converted_new_feature_set),
-        offline_map_supported=bool(16384 & converted_new_feature_set),
-    )
 
 
 @dataclass
@@ -501,7 +373,6 @@ class Status(RoborockBase):
     battery: int | None = None
     clean_time: int | None = None
     clean_area: int | None = None
-    square_meter_clean_area: float | None = None
     error_code: RoborockErrorCode | None = None
     map_present: int | None = None
     in_cleaning: RoborockInCleaning | None = None
@@ -548,26 +419,36 @@ class Status(RoborockBase):
     dss: int | None = None
     common_status: int | None = None
     corner_clean_mode: int | None = None
-    error_code_name: str | None = None
-    state_name: str | None = None
-    water_box_mode_name: str | None = None
-    fan_power_options: list[str] = field(default_factory=list)
-    fan_power_name: str | None = None
-    mop_mode_name: str | None = None
 
-    def __post_init__(self) -> None:
-        self.square_meter_clean_area = round(self.clean_area / 1000000, 1) if self.clean_area is not None else None
-        if self.error_code is not None:
-            self.error_code_name = self.error_code.name
-        if self.state is not None:
-            self.state_name = self.state.name
-        if self.water_box_mode is not None:
-            self.water_box_mode_name = self.water_box_mode.name
-        if self.fan_power is not None:
-            self.fan_power_options = self.fan_power.keys()
-            self.fan_power_name = self.fan_power.name
-        if self.mop_mode is not None:
-            self.mop_mode_name = self.mop_mode.name
+    @property
+    def square_meter_clean_area(self) -> float | None:
+        return round(self.clean_area / 1000000, 1) if self.clean_area is not None else None
+
+    @property
+    def error_code_name(self) -> str | None:
+        return self.error_code.name if self.error_code else None
+
+    @property
+    def state_name(self) -> str | None:
+        return self.state.name if self.state else None
+
+    @property
+    def water_box_mode_name(self) -> str | None:
+        return self.water_box_mode.name if self.water_box_mode else None
+
+    @property
+    def fan_power_options(self) -> list[str]:
+        if self.fan_power is None:
+            return []
+        return list(self.fan_power.keys())
+
+    @property
+    def fan_power_name(self) -> str | None:
+        return self.fan_power.name if self.fan_power else None
+
+    @property
+    def mop_mode_name(self) -> str | None:
+        return self.mop_mode.name if self.mop_mode else None
 
     def get_fan_speed_code(self, fan_speed: str) -> int:
         if self.fan_power is None:
@@ -584,6 +465,18 @@ class Status(RoborockBase):
             raise RoborockException("Attempted to get mop_mode before status has been updated.")
         return self.mop_mode.as_dict().get(mop_mode)
 
+    @property
+    def current_map(self) -> int | None:
+        """Returns the current map ID if the map is present."""
+        if self.map_status is not None:
+            map_flag = self.map_status >> 2
+            if map_flag != NO_MAP:
+                return map_flag
+        return None
+
+    def __repr__(self) -> str:
+        return _attr_repr(self)
+
 
 @dataclass
 class S4MaxStatus(Status):
@@ -678,6 +571,20 @@ class S8MaxvUltraStatus(Status):
     mop_mode: RoborockMopModeS8MaxVUltra | None = None
 
 
+@dataclass
+class Saros10RStatus(Status):
+    fan_power: RoborockFanSpeedSaros10R | None = None
+    water_box_mode: RoborockMopIntensitySaros10R | None = None
+    mop_mode: RoborockMopModeSaros10R | None = None
+
+
+@dataclass
+class Saros10Status(Status):
+    fan_power: RoborockFanSpeedSaros10 | None = None
+    water_box_mode: RoborockMopIntensitySaros10 | None = None
+    mop_mode: RoborockMopModeSaros10 | None = None
+
+
 ModelStatus: dict[str, type[Status]] = {
     ROBOROCK_S4_MAX: S4MaxStatus,
     ROBOROCK_S5_MAX: S5MaxStatus,
@@ -701,6 +608,8 @@ ModelStatus: dict[str, type[Status]] = {
     ROBOROCK_QREVO_MAXV: QRevoMaxVStatus,
     ROBOROCK_QREVO_PRO: P10Status,
     ROBOROCK_S8_MAXV_ULTRA: S8MaxvUltraStatus,
+    ROBOROCK_SAROS_10R: Saros10RStatus,
+    ROBOROCK_SAROS_10: Saros10Status,
 }
 
 
@@ -718,28 +627,30 @@ class ValleyElectricityTimer(RoborockBaseTimer):
 class CleanSummary(RoborockBase):
     clean_time: int | None = None
     clean_area: int | None = None
-    square_meter_clean_area: float | None = None
     clean_count: int | None = None
     dust_collection_count: int | None = None
     records: list[int] | None = None
     last_clean_t: int | None = None
 
-    def __post_init__(self) -> None:
+    @property
+    def square_meter_clean_area(self) -> float | None:
+        """Returns the clean area in square meters."""
         if isinstance(self.clean_area, list | str):
             _LOGGER.warning(f"Clean area is a unexpected type! Please give the following in a issue: {self.clean_area}")
-        else:
-            self.square_meter_clean_area = round(self.clean_area / 1000000, 1) if self.clean_area is not None else None
+            return None
+        return round(self.clean_area / 1000000, 1) if self.clean_area is not None else None
+
+    def __repr__(self) -> str:
+        """Return a string representation of the object including all attributes."""
+        return _attr_repr(self)
 
 
 @dataclass
 class CleanRecord(RoborockBase):
     begin: int | None = None
-    begin_datetime: datetime.datetime | None = None
     end: int | None = None
-    end_datetime: datetime.datetime | None = None
     duration: int | None = None
     area: int | None = None
-    square_meter_area: float | None = None
     error: int | None = None
     complete: int | None = None
     start_type: RoborockStartType | None = None
@@ -750,12 +661,20 @@ class CleanRecord(RoborockBase):
     wash_count: int | None = None
     map_flag: int | None = None
 
-    def __post_init__(self) -> None:
-        self.square_meter_area = round(self.area / 1000000, 1) if self.area is not None else None
-        self.begin_datetime = (
-            datetime.datetime.fromtimestamp(self.begin).astimezone(timezone.utc) if self.begin else None
-        )
-        self.end_datetime = datetime.datetime.fromtimestamp(self.end).astimezone(timezone.utc) if self.end else None
+    @property
+    def square_meter_area(self) -> float | None:
+        return round(self.area / 1000000, 1) if self.area is not None else None
+
+    @property
+    def begin_datetime(self) -> datetime.datetime | None:
+        return datetime.datetime.fromtimestamp(self.begin).astimezone(timezone.utc) if self.begin else None
+
+    @property
+    def end_datetime(self) -> datetime.datetime | None:
+        return datetime.datetime.fromtimestamp(self.end).astimezone(timezone.utc) if self.end else None
+
+    def __repr__(self) -> str:
+        return _attr_repr(self)
 
 
 @dataclass
@@ -769,44 +688,49 @@ class Consumable(RoborockBase):
     dust_collection_work_times: int | None = None
     cleaning_brush_work_times: int | None = None
     moproller_work_time: int | None = None
-    main_brush_time_left: int | None = None
-    side_brush_time_left: int | None = None
-    filter_time_left: int | None = None
-    sensor_time_left: int | None = None
-    strainer_time_left: int | None = None
-    dust_collection_time_left: int | None = None
-    cleaning_brush_time_left: int | None = None
-    mop_roller_time_left: int | None = None
 
-    def __post_init__(self) -> None:
-        self.main_brush_time_left = (
-            MAIN_BRUSH_REPLACE_TIME - self.main_brush_work_time if self.main_brush_work_time is not None else None
-        )
-        self.side_brush_time_left = (
-            SIDE_BRUSH_REPLACE_TIME - self.side_brush_work_time if self.side_brush_work_time is not None else None
-        )
-        self.filter_time_left = (
-            FILTER_REPLACE_TIME - self.filter_work_time if self.filter_work_time is not None else None
-        )
-        self.sensor_time_left = (
-            SENSOR_DIRTY_REPLACE_TIME - self.sensor_dirty_time if self.sensor_dirty_time is not None else None
-        )
-        self.strainer_time_left = (
-            STRAINER_REPLACE_TIME - self.strainer_work_times if self.strainer_work_times is not None else None
-        )
-        self.dust_collection_time_left = (
+    @property
+    def main_brush_time_left(self) -> int | None:
+        return MAIN_BRUSH_REPLACE_TIME - self.main_brush_work_time if self.main_brush_work_time is not None else None
+
+    @property
+    def side_brush_time_left(self) -> int | None:
+        return SIDE_BRUSH_REPLACE_TIME - self.side_brush_work_time if self.side_brush_work_time is not None else None
+
+    @property
+    def filter_time_left(self) -> int | None:
+        return FILTER_REPLACE_TIME - self.filter_work_time if self.filter_work_time is not None else None
+
+    @property
+    def sensor_time_left(self) -> int | None:
+        return SENSOR_DIRTY_REPLACE_TIME - self.sensor_dirty_time if self.sensor_dirty_time is not None else None
+
+    @property
+    def strainer_time_left(self) -> int | None:
+        return STRAINER_REPLACE_TIME - self.strainer_work_times if self.strainer_work_times is not None else None
+
+    @property
+    def dust_collection_time_left(self) -> int | None:
+        return (
             DUST_COLLECTION_REPLACE_TIME - self.dust_collection_work_times
             if self.dust_collection_work_times is not None
             else None
         )
-        self.cleaning_brush_time_left = (
+
+    @property
+    def cleaning_brush_time_left(self) -> int | None:
+        return (
             CLEANING_BRUSH_REPLACE_TIME - self.cleaning_brush_work_times
             if self.cleaning_brush_work_times is not None
             else None
         )
-        self.mop_roller_time_left = (
-            MOP_ROLLER_REPLACE_TIME - self.moproller_work_time if self.moproller_work_time is not None else None
-        )
+
+    @property
+    def mop_roller_time_left(self) -> int | None:
+        return MOP_ROLLER_REPLACE_TIME - self.moproller_work_time if self.moproller_work_time is not None else None
+
+    def __repr__(self) -> str:
+        return _attr_repr(self)
 
 
 @dataclass
@@ -817,19 +741,20 @@ class MultiMapsListMapInfoBakMaps(RoborockBase):
 
 @dataclass
 class MultiMapsListMapInfo(RoborockBase):
-    _ignore_keys = ["mapFlag"]
-
-    mapFlag: int
+    map_flag: int
     name: str
     add_time: Any | None = None
     length: Any | None = None
     bak_maps: list[MultiMapsListMapInfoBakMaps] | None = None
 
+    @property
+    def mapFlag(self) -> int:
+        """Alias for map_flag, returns the map flag as an integer."""
+        return self.map_flag
+
 
 @dataclass
 class MultiMapsList(RoborockBase):
-    _ignore_keys = ["mapFlag"]
-
     max_multi_map: int | None = None
     max_bak_map: int | None = None
     multi_map_count: int | None = None
@@ -861,11 +786,42 @@ class NetworkInfo(RoborockBase):
     rssi: int | None = None
 
 
+@dataclass
+class AppInitStatusLocalInfo(RoborockBase):
+    location: str
+    bom: str | None = None
+    featureset: int | None = None
+    language: str | None = None
+    logserver: str | None = None
+    wifiplan: str | None = None
+    timezone: str | None = None
+    name: str | None = None
+
+
+@dataclass
+class AppInitStatus(RoborockBase):
+    local_info: AppInitStatusLocalInfo
+    feature_info: list[int]
+    new_feature_info: int
+    new_feature_info_str: str
+    new_feature_info_2: int | None = None
+    carriage_type: int | None = None
+    dsp_version: int | None = None
+
+
 @dataclass
 class DeviceData(RoborockBase):
     device: HomeDataDevice
     model: str
     host: str | None = None
+    device_features: DeviceFeatures | None = None
+
+    @property
+    def product_nickname(self) -> RoborockProductNickname:
+        return SHORT_MODEL_TO_ENUM.get(self.model.split(".")[-1], RoborockProductNickname.PEARLPLUS)
+
+    def __repr__(self) -> str:
+        return _attr_repr(self)
 
 
 @dataclass
@@ -888,6 +844,7 @@ class FlowLedStatus(RoborockBase):
 class BroadcastMessage(RoborockBase):
     duid: str
     ip: str
+    version: bytes
 
 
 class ServerTimer(NamedTuple):
@@ -949,11 +906,15 @@ class RoborockProduct(RoborockBase):
     agreements: list | None = None
     cardspec: str | None = None
     plugin_pic_url: str | None = None
-    products_specification: RoborockProductSpec | None = None
 
-    def __post_init__(self):
+    @property
+    def product_nickname(self) -> RoborockProductNickname | None:
         if self.cardspec:
-            self.products_specification = RoborockProductSpec.from_dict(json.loads(self.cardspec).get("data"))
+            return RoborockProductSpec.from_dict(json.loads(self.cardspec).get("data"))
+        return None
+
+    def __repr__(self) -> str:
+        return _attr_repr(self)
 
 
 @dataclass
